
/*
Controle de tlcommandes ON(1) et OFF(0) choisit par un CODE
Les donnes sont reues par le port srie sous la forme d'une trame compos : 
'C'+'CODE'+'M'ou'A' ' avec CODE de 0  15
Une fois le code choisit il faut mettre en marche par MARCHE
*/

#include <16F88.h>


#device adc=8

#FUSES NOWDT                    //No Watch Dog Timer
#FUSES INTRC_IO                 //Internal RC Osc, no CLKOUT
#FUSES NOPUT                    //No Power Up Timer
#FUSES MCLR                     //Master Clear pin enabled
#FUSES NOBROWNOUT               //No brownout reset
#FUSES NOLVP                    //No low voltage prgming, B3(PIC16) or B5(PIC18) used for I/O
#FUSES NOCPD                    //No EE protection
#FUSES NOWRT                    //Program memory not write protected
#FUSES NODEBUG                  //No Debug mode for ICD
#FUSES NOPROTECT                //Code not protected from reading
#FUSES FCMEN                    //Fail-safe clock monitor enabled
#FUSES IESO                     //Internal External Switch Over mode enabled

#use delay(clock=8000000)
#use rs232(baud=9600,parity=N,xmit=PIN_B5,rcv=PIN_B2,bits=8,errors)


#include <stdlib.h>

#define MARCHE PIN_B0 //tranistor marche  relier entre 0 et COM
#define ON PIN_A4 //inter pour ON
#define OFF PIN_A6 //inter pour OFF

const int Nbcodemax=16;
const int TailleTrameRX = 64;

char trame_rx[TailleTrameRX]; //tableau  de reception de trame intisiali avec des 'i'
int8 tabcode[Nbcodemax];//tableau des valeurs des codes : num code = indice du tableau, valeur = 0 ou 1

void TXTELECOM();
void init();


#int_RDA
void  RDA_isr(void) 
{
int il;

   disable_interrupts(INT_RDA);
   gets(trame_rx);
   //printf("1");for(il=0;il<TailleTrameRX;il++)printf("%c",trame_rx[il]);
   //printf("2");
   TXTELECOM();//printf("2");
   enable_interrupts(INT_RDA);

}

void TXTELECOM(){

/*
envoi les commandes d'allumage ou d'extinction vers les sorties  partir des 
donnes reues dans trame_rx[].
Exemple de donnes reues : C1AC2MC3AC4AC5M (code1  ARRET(0) , code2  MARCHE(1)...)


algo : 
lire successivement les valeurs de trame_rx,identifie les 'C', 
identifier le nombre (1 ou 2), identifier si Arret 'A' = 0 et marche 'M'=1
methode choisie : lire trame_rx et sauvegarder dans une tableau tabcode[0..15]
les valeurs 0 ou 1, l'indice du tableau correspond au numero du code
rmq : la trame est pleine de 'X'  vide ! attention  la rinitialiser 
apres dcodage pour prparer la trame-rx suivante.

*/

int il,il2; //indice de boucle

//lecture de decodage de la trame
il=0;il2=0;
while(trame_rx[il]=='C')
{
   printf("K");printf("%u",il);
   //printf("%c",trame_rx[il]);
            
   if (trame_rx[il+3]=='A') tabcode[il2]=0;
   else if (trame_rx[il+3]=='M') tabcode[il2]=1;   
   il=il+4;
   il2=il/4;//printf("%u",il2);
}//fin while //decodage trame_rx
printf("L");
//for(il=0;il<Nbcodemax;il++)printf("%u%u",il,tabcode[il]);

//controle des sorties par lecture du tableau tabcode


output_high(MARCHE);//mise en marche telecommande
for (il=0;il<Nbcodemax;il++)
{
   il=il & 127;//masque pour isoler PINA0  PINA3
   output_a(il);                        //configure le CODE
   if (tabcode[il]==0) 
   {  output_low(ON);  //mise  OFF
      output_high(OFF);
   }
   if (tabcode[il]==1) 
   {  
      output_low(OFF);
      output_high(ON);    //mise  ON
   }
   printf("J%u",tabcode[il]);
   delay_ms(100);                           //attente
}//fin for
output_low(MARCHE);//arret telecommande

delay_ms(1000);

}//fin TXTELECOM

void init()
{

int i;

   setup_adc_ports(NO_ANALOGS|VSS_VDD);
   setup_adc(ADC_OFF);
   setup_spi(SPI_SS_DISABLED);
   setup_timer_0(RTCC_INTERNAL|RTCC_DIV_1);
   setup_timer_1(T1_DISABLED);
   setup_timer_2(T2_DISABLED,0,1);
   setup_comparator(NC_NC_NC_NC);
   setup_vref(FALSE);
   enable_interrupts(INT_RDA);
   enable_interrupts(GLOBAL);
   setup_oscillator(OSC_8MHZ|OSC_INTRC);
   
   
   set_tris_a(0b00100000);
   set_tris_b(0b11111110);
   
   for(i=0;i<TailleTrameRX;i++) trame_rx[i]='X';//initialise avec des 'X'
   for(i=0;i<NbCodemax;i++) tabcode[i]=0;//initialise le tableau  0
}//fin init

void main()
{


init();

printf("Z");
//for(il=0;il<TailleTrameRX;il++)printf("%c",trame_rx[il]);
while(1){
//programme par interruption
 
   
}//fin while 
}
